diff mbox series

[PATCHv5,bpf-next,1/7] bpf: Allow trampoline re-attach for tracing and lsm programs

Message ID 20210414195147.1624932-2-jolsa@kernel.org
State New
Headers show
Series [PATCHv5,bpf-next,1/7] bpf: Allow trampoline re-attach for tracing and lsm programs | expand

Commit Message

Jiri Olsa April 14, 2021, 7:51 p.m. UTC
Currently we don't allow re-attaching of trampolines. Once
it's detached, it can't be re-attach even when the program
is still loaded.

Adding the possibility to re-attach the loaded tracing and
lsm programs.

Fixing missing unlock with proper cleanup goto jump reported
by Julia.

Reported-by: kernel test robot <lkp@intel.com>
Reported-by: Julia Lawall <julia.lawall@lip6.fr>
Acked-by: Toke Høiland-Jørgensen <toke@redhat.com>
Signed-off-by: Jiri Olsa <jolsa@kernel.org>
---
 kernel/bpf/syscall.c    | 23 +++++++++++++++++------
 kernel/bpf/trampoline.c |  4 ++--
 2 files changed, 19 insertions(+), 8 deletions(-)

Comments

Andrii Nakryiko April 15, 2021, 11:22 p.m. UTC | #1
On Wed, Apr 14, 2021 at 5:44 PM Jiri Olsa <jolsa@kernel.org> wrote:
>

> Currently we don't allow re-attaching of trampolines. Once

> it's detached, it can't be re-attach even when the program

> is still loaded.

>

> Adding the possibility to re-attach the loaded tracing and

> lsm programs.

>

> Fixing missing unlock with proper cleanup goto jump reported

> by Julia.

>

> Reported-by: kernel test robot <lkp@intel.com>

> Reported-by: Julia Lawall <julia.lawall@lip6.fr>

> Acked-by: Toke Høiland-Jørgensen <toke@redhat.com>

> Signed-off-by: Jiri Olsa <jolsa@kernel.org>

> ---


Acked-by: Andrii Nakryiko <andrii@kernel.org>


>  kernel/bpf/syscall.c    | 23 +++++++++++++++++------

>  kernel/bpf/trampoline.c |  4 ++--

>  2 files changed, 19 insertions(+), 8 deletions(-)

>


[...]
KP Singh April 19, 2021, 10:47 p.m. UTC | #2
On Fri, Apr 16, 2021 at 1:22 AM Andrii Nakryiko
<andrii.nakryiko@gmail.com> wrote:
>

> On Wed, Apr 14, 2021 at 5:44 PM Jiri Olsa <jolsa@kernel.org> wrote:

> >

> > Currently we don't allow re-attaching of trampolines. Once

> > it's detached, it can't be re-attach even when the program

> > is still loaded.

> >

> > Adding the possibility to re-attach the loaded tracing and

> > lsm programs.

> >

> > Fixing missing unlock with proper cleanup goto jump reported

> > by Julia.

> >

> > Reported-by: kernel test robot <lkp@intel.com>

> > Reported-by: Julia Lawall <julia.lawall@lip6.fr>

> > Acked-by: Toke Høiland-Jørgensen <toke@redhat.com>

> > Signed-off-by: Jiri Olsa <jolsa@kernel.org>

> > ---

>

> Acked-by: Andrii Nakryiko <andrii@kernel.org>


Thanks!

Acked-by: KP Singh <kpsingh@kernel.org>
diff --git a/kernel/bpf/trampoline.c b/kernel/bpf/trampoline.c
index 1f3a4be4b175..205c2cc36ad7 100644
--- a/kernel/bpf/trampoline.c
+++ b/kernel/bpf/trampoline.c
@@ -414,7 +414,7 @@  int bpf_trampoline_link_prog(struct bpf_prog *prog, struct bpf_trampoline *tr)
 	tr->progs_cnt[kind]++;
 	err = bpf_trampoline_update(tr);
 	if (err) {
-		hlist_del(&prog->aux->tramp_hlist);
+		hlist_del_init(&prog->aux->tramp_hlist);
 		tr->progs_cnt[kind]--;
 	}
 out:
@@ -437,7 +437,7 @@  int bpf_trampoline_unlink_prog(struct bpf_prog *prog, struct bpf_trampoline *tr)
 		tr->extension_prog = NULL;
 		goto out;
 	}
-	hlist_del(&prog->aux->tramp_hlist);
+	hlist_del_init(&prog->aux->tramp_hlist);
 	tr->progs_cnt[kind]--;
 	err = bpf_trampoline_update(tr);
 out:
diff mbox series

Patch

diff --git a/kernel/bpf/syscall.c b/kernel/bpf/syscall.c
index fd495190115e..941ca06d9dfa 100644
--- a/kernel/bpf/syscall.c
+++ b/kernel/bpf/syscall.c
@@ -2648,14 +2648,25 @@  static int bpf_tracing_prog_attach(struct bpf_prog *prog,
 	 *   target_btf_id using the link_create API.
 	 *
 	 * - if tgt_prog == NULL when this function was called using the old
          *   raw_tracepoint_open API, and we need a target from prog->aux
          *
          * The combination of no saved target in prog->aux, and no target
          * specified on load is illegal, and we reject that here.
 	 */
 	if (!prog->aux->dst_trampoline && !tgt_prog) {
-		err = -ENOENT;
-		goto out_unlock;
+		/*
+		 * Allow re-attach for TRACING and LSM programs. If it's
+		 * currently linked, bpf_trampoline_link_prog will fail.
+		 * EXT programs need to specify tgt_prog_fd, so they
+		 * re-attach in separate code path.
+		 */
+		if (prog->type != BPF_PROG_TYPE_TRACING &&
+		    prog->type != BPF_PROG_TYPE_LSM) {
+			err = -EINVAL;
+			goto out_unlock;
+		}
+		btf_id = prog->aux->attach_btf_id;
+		key = bpf_trampoline_compute_key(NULL, prog->aux->attach_btf, btf_id);
 	}
 
 	if (!prog->aux->dst_trampoline ||
@@ -2648,14 +2648,25 @@  static int bpf_tracing_prog_attach(struct bpf_prog *prog,
 	 *
 	 * - if tgt_prog == NULL when this function was called using the old
-         *   raw_tracepoint_open API, and we need a target from prog->aux
-         *
-         * The combination of no saved target in prog->aux, and no target
-         * specified on load is illegal, and we reject that here.
+	 *   raw_tracepoint_open API, and we need a target from prog->aux
+	 *
+	 * - if prog->aux->dst_trampoline and tgt_prog is NULL, the program
+	 *   was detached and is going for re-attachment.
 	 */
 	if (!prog->aux->dst_trampoline && !tgt_prog) {